/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef APTLISTITEM_H
#define APTLISTITEM_H

#include <klistview.h>
#include <qmap.h>
#include <qdatetime.h>
#include <qpixmap.h>

/**
@author Sylvain Joyeux
*/

class DLListItem : public KListViewItem
{
	// For double buffering purposes
	bool m_needsRepaint;
	QPixmap m_pixmap;

	void updateContents();

  int m_state;
  bool m_showall;
  QString m_description, m_dlinfo;
  QStringList m_text;

  int m_current;
  int m_size;
  /** True if the progress bar has to be drawn */
  bool m_progress;
  /** The progress bar geometry */
  QRect m_prggeometry;

  QTime m_timestamp;

  /** This structure is the configuration of the
   * paint engine behavior. It is used mainly in paintCell
   * and setState */
  struct Config
  {
    /** This is the text describing the state */
    QString text;

    /** State color. Used by paintCell to paint the state text */
    QColor color;
    /** Sort order */
    int order;
    /** If true, this item is visible even if m_showall is false. @see setShowAll */
    bool show_always;
    /** Set to true if this state uses the progress bar */
    bool has_progress;


    /** Default constructor for QMap */
    Config()
      : order(-1), show_always(true), has_progress(false)  {}
    Config(QString t, QColor c, int o, bool sa, bool prg)
      : text(t), color(c), order(o), show_always(sa), has_progress(prg) {}
  };
  /** Mapping between Config structures and the state
   * number */
  static QMap<int, Config> s_config;

  /** Adds a new config object to s_config, using the KConfig object to load
  * the color, allowing us to make colors configurable  */
  static void loadConfigItem(KConfig* c, int state, const QString& key,
     const QColor& def_color, int order, bool show_always, bool has_progress);
  static void loadConfig();

  void computeHeight();
  virtual void setup();

	int getTabLength(const QFontMetrics& fm) const;

public:
  DLListItem(KListView* view, const QString& description, const QString& moreinfo,
    int size, bool showall);
  ~DLListItem();

  /** Sets the progression values. The progress bar is only
   * shown if the current state's config tells us to do so
   *
   * @param current the current value of the indicator
   * @param size the maximum value
   */
  void setProgression(int current, int max);

  /** @return the current state of this item */
  int getState() const;
  /** Sets the state of this item. The color and
   * display text are updated accordingly. If showAll has been
   * set to false, the item can be hidden or shown according
   * to the config of \c state */
  void setState(int state);


  void setDescription(const QString& desc);
  void setDLInfo(const QString& dlinfo);
  void setSize(unsigned long size);

  /** Adds a line of text below the description */
  void addText(const QString& text);
  /** if showAll is set, this item is visible even
   * if the show_always flag of its state is false */
  void setShowAll(bool show);

  virtual void paintCell(QPainter* p, const QColorGroup& cg,
    int column, int width, int alignment);
  virtual int compare(QListViewItem* i, int col, bool ascending) const;
	virtual int width(const QFontMetrics& fm, const QListView* list, int c) const;
};

#endif
