/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GETWIDGET_H
#define GETWIDGET_H

#include <kpkgmanagerd/getclient.h>
#include "getwidgetui.h"

class DLListItem;
class KListView;
class QListViewItem;

/**
 * @author Sylvain Joyeux
 */
class GetWidget : public GetWidgetUI, public GetClient
{
  Q_OBJECT

  KListView* m_list;

  struct DLPendingItem
  {
    QString desc, dlinfo;
    unsigned long size;
    QStringList text;
  };
  QMap<QString, DLPendingItem> m_pending;

  typedef QMap<QString, DLListItem*> Items;
  Items m_items;

  bool m_showall;
  bool m_active;
  bool m_taskHasBegun;
	bool m_taskHasStats;

  bool m_cancel;
  bool m_hasError;
  QString m_taskEndMsgError, m_taskEndMsgSuccess;

  int m_totalItems;

  /*QStringList m_stages;
  QStringList::ConstIterator m_curstage;*/
  void setActive(bool is_active);
  void endCleanup();

public:
  GetWidget(QWidget *parent = 0, const char *name = 0);
  ~GetWidget();

  bool isActive() const;
  bool endStatus() const;

  void displayGlobalAction(const QString& description);
  void discardGlobalAction();

public slots:
  virtual void update();
  virtual void install(const QString& commands);
  virtual void upgrade();
  virtual void distupgrade();

  virtual void cancel();

  void setShowAll(bool show);

signals:
  void active(bool is_active);
  void setStatusText(const QString& text);
  void userInteractionBegin(const QString& message = "");
  void userInteractionEnd(const QString& message = "");

protected:
  void beginTask(const QString& description,
    const QString& end_message_success, const QString& end_message_errors, bool has_stats = true);

  virtual void beginTask();
  virtual void endTask();

  virtual void beginStage(int category, QString newstage);

  virtual void newItem(QString key, QString desc, QString dlinfo, int size);
  virtual void setItemState(QString key, int state);
  void addItemText(const QString& key, const QString& message);
  virtual void itemProgression(QString key, int current, int total);

  virtual void questionYN(QString message);
  virtual void questionText(QString message);

  virtual void conffile(QString file);

  virtual void stats(APT::Stats stats);

  virtual void overallProgression(int curItem, int totalItems,
    double curBytes, double totalBytes, double cps);

  virtual void error(QString key, QString message);
  virtual void warning(QString key, QString message);
  virtual void info(QString key, QString message);

  virtual void subwin(int id);

  virtual void exit();

private slots:
  void itemExecuted(QListViewItem* item);
};

#endif
