/*
 * Copyright (C) 2004 Sylvain Joyeux <sylvain.joyeux@m4x.org>
 */

#ifndef _KPKGMANAGER_H
#define _KPKGMANAGER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kmainwindow.h>

class KListView;
class KAction;
class KToggleAction;
class GetWidget;
class KSystemTray;
class PkgManagerDCOP;
class KConfig;

/**
 * @short Application Main Window
 * @author Sylvain Joyeux <sylvain.joyeux@m4x.org>
 * @version 0.1
 */
class KPkgManager : public KMainWindow
{
  friend class PkgManagerDCOP;

  Q_OBJECT

  GetWidget* m_central;
  KAction* m_actContinue, * m_actCancel;
  KToggleAction* m_actStatus, * m_actShowall;

  KAction* m_actInstall, *m_actUpgrade, *m_actDistUpgrade, *m_actUpdate;
  KSystemTray* m_systray;
  int m_autohide;
  bool m_userInteractionSection;

  PkgManagerDCOP* m_dcop;

  void setupActions();
  void setupSystray();
  void initAutoHide();

  void buildSystrayTip(const QString& status);

public:
  KPkgManager();
  virtual ~KPkgManager();

  GetWidget* getGetWidget() const;

public slots:
  void configureNotifications();
  void install();
  void update();
  void upgrade();
  void distupgrade();

protected slots:
  void isActive(bool is_active);
  void cancelCurTask();
  void setGetWidgetStatus(const QString& message);
  void setShowAll(bool show);

  void resetAutoHide();

  void userInteractionBegin(const QString& message);
  void userInteractionEnd(const QString& message);

  void updateConfiguration();
  void configure();

protected:
  virtual bool queryExit();
  virtual bool queryClose();

  virtual void showEvent(QShowEvent* e);
  virtual void hideEvent(QHideEvent* e);

  virtual void saveGlobalProperties(KConfig* session);
  virtual void readGlobalProperties(KConfig* session);
};

#endif // _KPKGMANAGER_H_

