/*
 * Copyright (C) 2004 Sylvain Joyeux <sylvain.joyeux@m4x.org>
 */

#include "config.h"

#include "kpkgmanager.h"
#include "getwidget.h"

#include <qfile.h>
#include <qtextstream.h>

#include <kuniqueapplication.h>
#include <dcopclient.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kwin.h>
#include <kstartupinfo.h>
#include <ksystemtray.h>

static const char description[] =
    I18N_NOOP("A package management tool for KDE");

static const char version[] = VERSION;

static KCmdLineOptions options[] =
{
  { "c", 0, 0 },
  { "command <command>", I18N_NOOP("Commands to execute"), 0 },
  { "q", 0, 0 },
  { "quiet", I18N_NOOP("Be as less intrusive as possible"), false },
  { "+[file]", I18N_NOOP("Read the commands to execute from this file (one per line)"), 0 },
  { "", I18N_NOOP("The command is a space-separated list of packages, with:\n"
                  "\ta + in front of the package name to install it\n"
                  "\ta - in front of the package name to remove it\n"
                  "\t/distribution (/testing, /unstable, ...) after the package name\n"
                  "\t\tin order to select a particular distribution\n"
                  "\t=version after the package name to select a particular version\n"), 0 },
  { "", I18N_NOOP("For instance,\n"
                  "\t+kpkgmanager=0.1-1 installs the version 0.1-1 of kpkgmanager\n"
                  "\t-aptitude removes the package aptitude"), 0 },
//    { "+[URL]", I18N_NOOP( "Document to open." ), 0 },
    KCmdLineLastOption
};

int main(int argc, char **argv)
{
    KAboutData about("kpkgmanager", I18N_NOOP("KPkgManager"), version, description,
                     KAboutData::License_GPL, "(C) 2004 Sylvain Joyeux", 0, 0, "sylvain.joyeux@m4x.org");
    about.addAuthor( "Sylvain Joyeux", 0, "sylvain.joyeux@m4x.org" );
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions( options );

    KApplication app;

    // Parse the command line
    // If one kpkgmanager is already running, we forward the commands
    // via DCOP.
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    QStringList install_commands;
    QString command = args->getOption("command");
    if (!command.isNull())
      install_commands += command;

    if (args->count())
    {
      QString command_file = args->url(0).path();
      QFile file(command_file);
      if (!file.open(IO_ReadOnly))
      {
        KMessageBox::error(0, i18n("Unable to open the command file '%1'").arg(command_file));
        return 1;
      }

      QTextStream stream(&file);
      while(!stream.atEnd())
        install_commands += stream.readLine();
    }

    bool is_quiet = args->isSet("quiet");


    // Check if another kpkgmanager is running
    {
      DCOPClient* client = app.dcopClient();
      QCString id = client -> registerAs("kpkgmanager", false);
      if (id.isNull())
      {
        KMessageBox::error(0, i18n("Unable to attach to DCOP server, aborting"));
        return 1;
      }

      if (id != "kpkgmanager")
      {
        const QString app = "kpkgmanager";

        DCOPRef iface("kpkgmanager", "PkgManagerIface");

        // forward the commands (if any)
        if (!install_commands.empty())
        { iface.call("install", install_commands, is_quiet); }
        else
        {
          DCOPRef mainwin("kpkgmanager", "kpkgmanager-mainwindow#1");

          if (!is_quiet) {
            mainwin.call("show");
            int winId = mainwin.call("getWinID");

            int current_desktop = KWin::currentDesktop();
            KWin::setOnDesktop(winId, current_desktop);
            KWin::forceActiveWindow(winId);
          }
        }

        KStartupInfo::appStarted();
        return 0;
      }
    }

    KPkgManager *mainWin = 0;
    if(kapp->isRestored() && KMainWindow::canBeRestored(1))
    {
			mainWin = new KPkgManager;
			mainWin->restore(1);
    }

    if (!mainWin)
    {
      mainWin = new KPkgManager;
    }
		app.setMainWidget( mainWin );

    GetWidget* get_client = mainWin -> getGetWidget();
    if (!install_commands.empty())
      get_client->install(install_commands.join(" "));

    args->clear();

    // mainWin has WDestructiveClose flag by default, so it will delete itself.
    return app.exec();
}

