/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef STATSDIALOG_H
#define STATSDIALOG_H

#include "statswidget.h"
#include <kdialog.h>

#include <qvaluelist.h>
#include <kurl.h>

/**
@author Sylvain Joyeux
*/

namespace APT
{
  class Stats;
};

class KAction;
class KToolBar;
class KListViewItem;
class KHTMLPart;
class KLineEdit;
class StatsWidget;
class QListViewItem;
class KActionCollection;

/** The dialog which pops-up when we request the confirmation of
 * an action.
 *
 * It lists the packages involved (installed, removed), allows
 * package browsing using kio-apt if it is installed, and (eventually)
 * will allow installing recommended packages, keeping upgraded
 * packages, ...
 */
class StatsDialog : public StatsWidget
{
  Q_OBJECT

  KHTMLPart* m_html;
  KActionCollection* m_actionCollection;

  KLineEdit* m_showLineEdit;

  KAction* m_actBack, * m_actForward, * m_actShow;

  bool m_showHTML;

public:
  struct Item
  {
    QString package;
    int state;
    unsigned long size;
    Item() :size(0) {};
    Item(QString p, int st, unsigned long s)
      : package(p), state(st), size(s) {}
  };
  typedef QValueList<Item> ItemList;

  typedef QValueList<KURL> URLList;
  /// The URL history used when the user is browsing
  URLList m_urlHistory;
  /// Current item in history
  URLList::ConstIterator m_urlCurrent;

private:
  /** The items currently in the tree view */
  ItemList m_itemList;

  /** Fills the labels (byte counts, counts of package installed)
   * @arg event the event from kpkgmanagerd */
  void fillLabels(const APT::Stats& event);

  enum ActionMode
  {
    DoNothing = 0, // There's nothing to be done
    Normal = 1, // Normal (harmless) mode of action
    Dangerous = 2 // The action requested is potentially dangerous
  };

  /** Fills the tree view and the item lists
   * with the data from kpkgmanaged
   * @arg event the event from kpkgmanagerd
   * @return the mode of action, see ActionMode for values*/
  int fillTree(const APT::Stats& event);

  /** Sets up the actions, and plug the browsing actions
   * into a toolbar
   * @arg toolbar the toolbar we should plug the actions in */
  void setupActions(KToolBar* toolbar);
  /** Sets up the layout. Basically, it'll hide or show the
   * browsing part of the dialog according to what will be done */
  void setupLayout();
  /** Saves the layout size */
  void saveLayout();

  /** Enabled or disables the forward/previous actions according
   * to what is the current URL and the state of the URL history */
  void setBrowseActionsState();

  /** Checks for kio-apt presence
   * @return true if the apt:/ protocol can be handled and
   * false otherwise */
  static bool hasKIOApt();


public:
  StatsDialog(QWidget *parent = 0, const char *name = 0);
  ~StatsDialog();

  /** Run the dialog in modal state */
  bool exec(const APT::Stats& event);

  const ItemList& getItems() const;

public slots:
  void showPackage(const QString& package);

private slots:
  void packageExecuted(QListViewItem*);
/*  void showDetails();
  void showDetails(bool show);*/

  void openURL(const KURL& url);
  void urlForward();
  void urlBack();
};

#endif
