/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getdaemon.h"

#include <kapplication.h>
#include <libkdeapt/task.h>

#include <kdebug.h>

GetDaemon::GetDaemon()
 : DCOPObject("Get")
{  }

GetDaemon::~GetDaemon() {}

bool GetDaemon::install(QStringList query)
{ return APT::Get::install(query); }
bool GetDaemon::update()
{ return APT::Get::update(); }
bool GetDaemon::upgrade(bool distupgrade)
{ return APT::Get::upgrade(distupgrade); }

void GetDaemon::setQueryAnswer(QVariant answer)
{ APT::Get::setQueryAnswer(answer); }
bool GetDaemon::needsAnswer()
{ return APT::Get::needsAnswer(); }


void GetDaemon::beginTask(APT::BeginTask* /*event*/)
{ emit GetIface::beginTask(); }
void GetDaemon::endTask(APT::EndTask* event)
{
  APT::Task* task = event->task;
  task->wait();
  delete task;
  emit GetIface::endTask();
}

void GetDaemon::beginStage(APT::BeginStage* event)
{ emit GetIface::beginStage(event->category,event->stage); }

void GetDaemon::newItem(APT::NewItem* event)
{ emit GetIface::newItem(event->key, event->description, event->moreinfo, event->size); }
void GetDaemon::setItemState(APT::SetItemState* event)
{ emit GetIface::setItemState(event -> key, event->state); }
void GetDaemon::itemProgression(APT::ItemProgression* event)
{ emit GetIface::itemProgression(event->key, event->current, event->total); }

void GetDaemon::questionYN(APT::Prompt* event)
{ emit GetIface::questionYN(event->question); }
void GetDaemon::questionText(APT::Prompt* event)
{ emit GetIface::questionText(event->question); }

void GetDaemon::conffile(APT::Conffile* event)
{ emit GetIface::conffile(event->file); }

void GetDaemon::stats(APT::Stats* event)
{ emit GetIface::stats(*event); }

void GetDaemon::overallProgression(APT::OverallProgression* event)
{ emit GetIface::overallProgression(event->curItem, event->totalItems,
    event->curBytes, event->totalBytes, event->cps); }

void GetDaemon::error(APT::Notify* event)
{ emit GetIface::error(event->key, event->message); }
void GetDaemon::warning(APT::Notify* event)
{ emit GetIface::warning(event->key, event->message); }
void GetDaemon::info(APT::Notify* event)
{ emit GetIface::info(event->key, event->message); }

void GetDaemon::subwin(APT::SubwinId* event)
{ emit GetIface::subwin(event -> id); }

void GetDaemon::cancelCurTask( bool wait )
{
  APT::Task* ctask = APT::Task::currentTask();
  if (!ctask) return;
  kdDebug() << "Canceling current task" << endl;
  ctask -> cancel();
  kdDebug() << "Sent cancel to the task" << endl;
  if (wait)
  {
    kdDebug() << "Waiting task termination" << endl;
    ctask -> wait();
  }

  kdDebug() << "Sending posted events" << endl;
  qApp -> sendPostedEvents();
}

void GetDaemon::quit( )
{
  cancelCurTask(true);
  qApp -> quit();
}

