/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KAPTMODULE_H
#define KAPTMODULE_H

#include <kdedmodule.h>
#include <dcopref.h>

class KProcess;

/** The KDED daemon. It gives access (or not) to
 * kpkgmanagerd
 *
 * From now, access control is very simplistic, but
 * it will eventually be used for secure access
 *
 * @author Sylvain Joyeux
 */
class KDEDPkgManager : public KDEDModule
{
  Q_OBJECT
  K_DCOP

  QString m_locker;
  QString m_error;
  KProcess* m_daemon;

  QCString getDCOPServer();

  void unlockPM();

public:

  KDEDPkgManager(const QCString& obj);
  virtual ~KDEDPkgManager();

  bool launchDaemon();
  void killDaemon();

  void idle();

  void setError(const QString err_string);

k_dcop:
  bool lock();
  void unlock();
  bool isAvailable();
  void cancel(bool wait);

  bool hasError();
  QString error();

  DCOPRef pmGet();

k_dcop_signals:
  /** Emitted when the package manager daemon is ready
   * to process calls */
  void pmIsReady();
  /** Emitted when the package manager daemon exits
   * unexpectedly */
  void pmExit();

private slots:
  void pmHasExited(KProcess *);
  void pmRegistered(const QCString& appId);
  void lockerRemoved(const QCString& appId);
};

#endif
