/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef APTACQPROGRESS_H
#define APTACQPROGRESS_H

#include <apt-pkg/acquire.h>

#include <qstring.h>
#include <qmap.h>
#include <qmutex.h>

namespace APT
{

  class Task;

  /** Callback object to the apt libraries download system
   * The APT events are translated into QCustomEvent objects
   * and sent via the m_parent Task object.
   *
   * @see progress.h for the event messages
   *
   * @author Sylvain Joyeux
   */
  class AcqProgress : public pkgAcquireStatus
  {
    /** Parent task */
    Task* m_parent;
    /** Next available id */
    int m_id;

    typedef QMap<int, QString> Keys;
    Keys m_keys;
    
    QMap<QString, bool> m_errors;

    QString NewItem(pkgAcquire::ItemDesc &Itm);

    QMutex m_mtx_stop;
    bool m_stop;

  public:
    AcqProgress(Task *parent);
    ~AcqProgress();

    void Start();
    bool MediaChange(string Media, string Drive);
    void IMSHit(pkgAcquire::ItemDesc &Itm);
    void Fetch(pkgAcquire::ItemDesc &Itm);
    void Done(pkgAcquire::ItemDesc &Itm);
    void Fail(pkgAcquire::ItemDesc &Itm);
    void Stop();
    bool Pulse(pkgAcquire *Owner);

	  void Reset();
    void Cancel();
  };

};

#endif
