/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kprocessrunner.h"

#include <qevent.h>
#include <qapplication.h>

#include <kdebug.h>

KProcessRunner::~KProcessRunner( )
{
}

KProcessRunner::KProcessRunner( KProcess * process )
  : m_process(process)
{
}


struct KProcessRunner::Command : public QCustomEvent
{
	enum Codes
  {
		KILL, // KILL signo
    START // START runmode comm
  };
	int cmd;
 
  static const int Type = QCustomEvent::User;
 
  int int_params[2];
  
  Command(int code, int param0, int param1)
      : QCustomEvent(Type)   
  {
    cmd = code;
    int_params[0] = param0;
    int_params[1] = param1;
  }
};

void KProcessRunner::customEvent( QCustomEvent * event )
{
  Command* cmd = dynamic_cast<Command *>(event);
  if (!cmd) return;

  if (cmd -> cmd == Command::START)
  {
    QMutexLocker locker(&m_mutex);
  	int runmode = cmd->int_params[0];
  	int comm = cmd->int_params[1];
    m_result = m_process -> start((KProcess::RunMode) runmode, (KProcess::Communication) comm);    
    m_post.wakeAll();
  }
  else if (cmd -> cmd == Command::KILL)
  {
    QMutexLocker locker(&m_mutex);
  	int signo = cmd->int_params[0];
    m_result = m_process -> kill(signo);
    m_post.wakeAll();
  }
}

bool KProcessRunner::start( KProcess::RunMode runmode, int comm )
{
  if (runmode == KProcess::Block) return false;
  
  QMutexLocker locker(&m_mutex);
  QApplication::postEvent(this, new Command(Command::START, runmode, (KProcess::Communication) comm));  
  m_post.wait(&m_mutex);
  return m_result;
}

bool KProcessRunner::kill( int signo )
{
  QApplication::postEvent(this, new Command(Command::KILL, signo, 0));
  
  QMutexLocker locker(&m_mutex);
  m_post.wait(&m_mutex);
  return m_result;
}

void KProcessRunner::wait( )
{
  connect(m_process, SIGNAL(processExited(KProcess*)),
       this, SLOT(exited(KProcess *)));
  
  QMutexLocker locker(&m_mutex);
  if (m_process -> isRunning())
    m_wait.wait(&m_mutex);    
}

void KProcessRunner::exited( KProcess * )
{
  disconnect(m_process, SIGNAL(processExited(KProcess*)),
       this, SLOT(exited(KProcess *)));
  
  QMutexLocker locker(&m_mutex);
  m_wait.wakeAll();
}

#include "kprocessrunner.moc"
