/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __kprocessrunner_h__
#define __kprocessrunner_h__

#include <qobject.h>

#include <kprocess.h>

#include <qmutex.h>
#include <qwaitcondition.h>

#include <signal.h>

class QCustomEvent;

/** Makes easy to run the KProcess class when being
 * outside the Qt thread. It *SHALL* be used outside
 * of the GUI thread, otherwise you'll end with
 * deadlocks
 */
class KProcessRunner : public QObject
{
	Q_OBJECT

  KProcess* m_process;

  class Command;
  void customEvent(QCustomEvent* event);

  bool m_result;
  QWaitCondition m_post, m_wait;
  QMutex m_mutex;

private slots:
  void exited(KProcess*);

public:
  /** Creates a KProcessRunner attached to \c process. The
   * KProcessRunner object is then ran using start() */
	KProcessRunner(KProcess* process);
	~KProcessRunner();

  /** Start the associated KProcess object in the Qt thread.
   * See KProcess::start for the arguments description
   * @return true if it started successfully, false otherwise */
  bool start(KProcess::RunMode runmode, int comm);
  /** Waits for the process termination */
	void wait();
  /** Kill the process */
  bool kill(int signal = SIGTERM);
};


#endif

