/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef APTPACKAGEMANAGER_H
#define APTPACKAGEMANAGER_H

#include <kprocio.h>
#include <qstringlist.h>
#include <apt-pkg/dpkgpm.h>

#include <qmutex.h>

class KProcessRunner;

namespace APT
{
  class Task;
  /** The package manager object used by the apt libraries
   * for access to the underlying package manager functionalities.
   *
   * This one calls dpkg and parses dpkg's output in order to provide
   * feedback and interaction within the GUI
   *
   * @author Sylvain Joyeux
   */
  class PackageManager : public pkgDPkgPM
  {
  protected:
      // For APT < 0.6
    virtual bool Go();
    // For APT >= 0.6
    virtual bool Go(int status_fd);

  public:
    PackageManager(pkgDepCache* cache);
    virtual ~PackageManager();
  };

  /** The class handling the dpkg process */
  class PackageManagerProcess : public KProcess
  {
    Q_OBJECT

    Task* m_parent;

    QMutex m_mtxRunner;
    KProcessRunner* m_runner;

    bool m_drop;
    QString m_path;

    QMap<QString, bool> m_items;

    QString m_inputBuffer;
    QStringList m_pendingOutput;

    static const int NullState = -1;
    /** The package which is currently treated by dpkg */
    QString m_package;
    /** The state of this package (unpacking, error, ...),
     * or NullState if we are not treating a package */
    int m_state;
    /** Messages that have been recorded while in
     * the current state.
     * They are sent when a state change occurs using
     * setState or resetState */
    void addMessage(const QString& message);

    void resetState();
    void setState(int state, const QString& new_package);

    void send(const QString& line);
    void write();

    void parseInput(const QString& line);

  public:
    PackageManagerProcess();

    /** Start the dpkg process, sending messages to the UI through
     * Task::currentTask
     * @args args the list of arguments to give to dpkg
     * @return true if dpkg has been run and exited normally */
    bool start(const QStringList& args);
    /** Run dpkg --configure-all */
    bool configureAll();
    /** Cancel the running dpkg */
    void cancel();

  public slots:
    /** The PackageManagerProcess runs in the Qt thread (because
     * KProcess has to). This slot is used to wait for the UI answer
     * to a Conffile message */
    void hasConffileAnswer(const QVariant& /*answer*/);
  protected slots:
    void receive(KProcess* process, char* buffer, int len);
    void sent(KProcess* proc);
  };

};

#endif
