/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef APTPACKAGESYSTEM_H
#define APTPACKAGESYSTEM_H

#include <apt-pkg/debsystem.h>

namespace APT
{
  class Task;
  /** Package system object used in interface to APT
   * It inherits debSystem, since its job is to replace the
   * package manager by our own (which will emit QEvent messages
   * during the package manager phase of the install
   *
   * @author Sylvain Joyeux
   */

  class PackageSystem : public debSystem
  {
  public:
    PackageSystem();
    ~PackageSystem();

    /** This package system score amongst the other package systems.
     * @return debSytem::score + 1, in order that this package system object
     * replaces the standardd debSystem object */
    virtual signed Score(Configuration const &Cnf);
    /** Creates the package manager object */
    virtual pkgPackageManager *CreatePM(pkgDepCache *Cache) const;

    /** This is the same as debSystem::CheckUpdates, but since
     * it is private, we don't have access here and though we have
     * to reimplement it */
    static bool checkUpdates();
  };
};

#endif
