/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef APT_COMMON_H

#include <qmutex.h>
#include <qstring.h>

#include <apt-pkg/acquire.h>

namespace APT
{
  class Task;
  class CacheFile;
  class Stats;
  class AcqProgress;

  class InstallPackages
  {
    Task* m_parent;
    CacheFile& m_cache;

    AcqProgress* m_acq;
    bool m_stop;
    QMutex m_mtxCancel;

    void buildStats(Stats* stats);
    void setPurgeDefault();

  public:
    InstallPackages(CacheFile& cache, Task* parent);
    ~InstallPackages();

    bool run(Stats* stats);
    void cancel();
    bool must_stop();

    static QString getKey(const pkgAcquire::ItemDesc& item)
    {
      QString description = QString::fromLatin1(item.Description.c_str());
      return getKey(description);
    }
    static QString getKey(const QString& description)
    {
      return description.section(" ", 2, 2);
    }
  };
};

#endif

