/***************************************************************************
 *   Copyright (C) 2004 by Sylvain Joyeux                                  *
 *   sylvain.joyeux@m4x.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "upgrade.h"

#include "common.h"
#include "cachefile.h"

#include <apt-pkg/error.h>
#include <apt-pkg/algorithms.h>

#include "../progress.h"
#include "../packagesystem.h"
#include "../packagemanager.h"

#include <kdebug.h>
#include <klocale.h>

namespace APT {

Upgrade::Upgrade(bool distupgrade, QObject* parent)
  : Task(parent),
    m_distupgrade(distupgrade), m_install(0), m_cancel(false)
{
}

Upgrade::~Upgrade()
{
  if (m_install)
    delete m_install;
}

void Upgrade::task( )
{
  CacheFile Cache(this);
  if (Cache.OpenForInstall() == false || Cache.CheckDeps() == false)
    return;

  bool ret;
  if (m_distupgrade)
  {
    beginStage(this, BeginStage::PreDownload, i18n("Computing dist-upgrade"));
    ret = pkgDistUpgrade(*Cache);
  }
  else
  {
    beginStage(this, BeginStage::PreDownload, i18n("Computing upgrade"));
    ret = pkgAllUpgrade(Cache);
  }

  if (ret == false)
  {
    //@todo show broken dependencies
    //ShowBroken(c1out,Cache,false);
    return;
  }

  {
    QMutexLocker lock(&m_mtxCancel);
    if (m_cancel) return;

    kdDebug() << "[Upgrade] Installing packages" << endl;
    if (m_install) delete m_install;
    m_install = new InstallPackages(Cache, this);
  }

  Stats* stats = new Stats;
  m_install -> run(stats);
}

void Upgrade::cancelTask()
{
  beginStage(this, BeginStage::Cancel, i18n("Stopping at user request ..."));

  QMutexLocker lock(&m_mtxCancel);;
  m_cancel = true;
  if (m_install) m_install -> cancel();
}

};
